require 'securerandom'
require 'bcrypt'

module ActsAsTokenable
  class Token < ActiveRecord::Base
    attr_accessor :token_hash

    before_update { false }

    belongs_to :tokenable, polymorphic: true

    validates :token_id, presence: true, uniqueness: true
    validates :token_hash, presence: true

    scope :active, -> { where('expires_at > ?', DateTime.now) }

    before_validation :generate_token_id
    before_validation :generate_token

    private
      def generate_token_id
        begin
          self.token_id = SecureRandom.hex(16)
        end while self.class.exists?(token_id: self.token_id)
      end

      def generate_token
        self.token = SecureRandom.urlsafe_base64(32)
        self.token_hash = BCrypt::Password.create(token, cost: cost)
      end

      def cost
        Rails.env.test? ? 1 : 10
      end
  end
end
